@@ -2,8 +2,8 @@
 #'@title  Data-Driven Nonparametric Shape Restriction and Parametric Model Specification Testing using Binscatter
 #'@description \code{binstest} implements binscatter-based hypothesis testing procedures for parametric functional
 #'             forms of and nonparametric shape restrictions on the regression function of interest, following the results
-#'             in \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_AER.pdf}{Cattaneo, Crump, Farrell and Feng (2023a)} and
-#'             \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_NonlinearBinscatter.pdf}{Cattaneo, Crump, Farrell and Feng (2023b)}.
+#'             in \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2024_AER.pdf}{Cattaneo, Crump, Farrell and Feng (2024a)} and
+#'             \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2024_NonlinearBinscatter.pdf}{Cattaneo, Crump, Farrell and Feng (2024b)}.
 #'             If the binning scheme is not set by the user,
 #'             the companion function \code{\link{binsregselect}} is used to implement binscatter in a
 #'             data-driven way and inference procedures are based on robust bias correction.
@@ -46,8 +46,9 @@
 #'@param  testshape2 a vector of null boundary values for hypothesis testing. Each number \code{a} in the vector
 #'                   corresponds to one boundary of a two-sided hypothesis test of the form
 #'                   \code{H0: sup_x |mu(x)-a|=0}.
-#'@param  lp an Lp metric used for (two-sided) parametric model specification testing and/or shape restriction testing. The default is \code{lp=Inf}, which
-#'           corresponds to the sup-norm of the t-statistic. Other options are \code{lp=q} for a positive integer \code{q}.
+#'@param  lp an Lp metric used for parametric model specification testing and/or shape restriction testing. The default is \code{lp=Inf}, which
+#'           corresponds to the sup-norm of the t-statistic. Other options are \code{lp=q} for a positive number \code{q>=1}. Note that
+#'           \code{lp=Inf} ("sup-norm") has to be used for testing one-sided shape restrictions.
 #'@param  bins a vector. If \code{bins=c(p,s)}, it sets the piecewise polynomial of degree \code{p} with \code{s} smoothness constraints
 #'             for data-driven (IMSE-optimal) selection of the partitioning/binning scheme.
 #'             The default is \code{bins=c(0,0)}, which corresponds to the piecewise constant.
@@ -93,7 +94,7 @@
 #'@param  dfcheck adjustments for minimum effective sample size checks, which take into account number of unique
 #'                values of \code{x} (i.e., number of mass points), number of clusters, and degrees of freedom of
 #'                the different stat models considered. The default is \code{dfcheck=c(20, 30)}.
-#'                See \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_Stata.pdf}{Cattaneo, Crump, Farrell and Feng (2023c)} for more details.
+#'                See \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2024_Stata.pdf}{Cattaneo, Crump, Farrell and Feng (2024c)} for more details.
 #'@param  masspoints how mass points in \code{x} are handled. Available options:
 #'                   \itemize{
 #'                   \item \code{"on"} all mass point and degrees of freedom checks are implemented. Default.
@@ -106,10 +107,10 @@
 #'@param  weights an optional vector of weights to be used in the fitting process. Should be \code{NULL} or
 #'                a numeric vector. For more details, see \code{\link{lm}}.
 #'@param  subset optional rule specifying a subset of observations to be used.
-#'@param  numdist  number of distinct for selection. Used to speed up computation.
+#'@param  numdist  number of distinct values for selection. Used to speed up computation.
 #'@param  numclust number of clusters for selection. Used to speed up computation.
-#'@param  estmethodopt a list of optional arguments used by \code{\link{rq}} (for quantile regression) or \code{\link{glm}} (for fitting generalized linear models).
-#'@param  ...      optional arguments to control bootstrapping if \code{estmethod="qreg"} and \code{vce="boot"}. See \code{\link{boot.rq}}.
+#'@param  estmethodopt a list of optional arguments used by \code{\link[quantreg]{rq}} (for quantile regression) or \code{\link{glm}} (for fitting generalized linear models).
+#'@param  ...      optional arguments to control bootstrapping if \code{estmethod="qreg"} and \code{vce="boot"}. See \code{\link[quantreg]{boot.rq}}.
 #'@return \item{\code{testshapeL}}{Results for \code{testshapel}, including: \code{testvalL}, null boundary values;
 #'                                 \code{stat.shapeL}, test statistics; and \code{pval.shapeL}, p-value.}
 #'        \item{\code{testshapeR}}{Results for \code{testshaper}, including: \code{testvalR}, null boundary values;
@@ -138,11 +139,11 @@
 #' Yingjie Feng (maintainer), Tsinghua University, Beijing, China. \email{fengyingjiepku@gmail.com}.
 #'
 #'@references
-#' Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2023a: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_AER.pdf}{On Binscatter}. Working Paper.
+#' Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2024a: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2024_AER.pdf}{On Binscatter}. American Economic Review 114(5): 1488-1514.
 #'
-#' Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2023b: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_NonlinearBinscatter.pdf}{Nonlinear Binscatter Methods}. Working Paper.
+#' Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2024b: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2024_NonlinearBinscatter.pdf}{Nonlinear Binscatter Methods}. Working Paper.
 #'
-#' Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2023c: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2023_Stata.pdf}{Binscatter Regressions}. Working Paper.
+#' Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2024c: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2024_Stata.pdf}{Binscatter Regressions}. Working Paper.
 #'
 #'@seealso \code{\link{binsreg}}, \code{\link{binsqreg}}, \code{\link{binsglm}}, \code{\link{binsregselect}}.
 #'
@@ -266,6 +267,15 @@ binstest <- function(y, x, w=NULL, data=NULL, estmethod="reg", family=gaussian()
   if (is.null(at))  at <- "mean"
 
   # change method name if needed
+  if (is.character(family))
+    family <- get(family, mode = "function", envir = parent.frame())
+  if (is.function(family))
+    family <- family()
+  if (is.null(family$family)) {
+    print(family)
+    stop("'family' not recognized")
+  }
+
   if (!is.null(family)) if (family$family!="gaussian" | family$link!="identity") {
     estmethod <- "glm"
   }
@@ -447,6 +457,14 @@ binstest <- function(y, x, w=NULL, data=NULL, estmethod="reg", family=gaussian()
     print("p<s not allowed.")
     exit <- 1
   }
+  if (lp<1) {
+    print("lp has to be no less than 1.")
+    exit <- 1
+  }
+  if (!is.null(testshapel) | !is.null(testshaper)) if (lp!=Inf) {
+    print("Sup-norm (lp=Inf) has to be used for testing one-sided restrictions.")
+    exit <- 1
+  }
   if (length(testshape)==2) if (testshape[1]<testshape[2]) {
     print("p<s not allowed.")
     exit <- 1
@@ -786,6 +804,7 @@ binstest <- function(y, x, w=NULL, data=NULL, estmethod="reg", family=gaussian()
         if (deriv == 0) pred.sha$fit <- linkinv(pred.sha$fit)
         if (deriv == 1) pred.sha$fit <- pred.sha.0 * pred.sha$fit
       } else {
+        basis.sha.0 <- basis.0
         basis.sha.1 <- basis.sha
         if (!is.null(eval.w)) {
           basis.sha.0 <- cbind(basis.0, outer(rep(1, nrow(basis.0)), eval.w))
@@ -925,6 +944,7 @@ binstest <- function(y, x, w=NULL, data=NULL, estmethod="reg", family=gaussian()
              if (deriv == 0) pred.mod$fit <- linkinv(pred.mod$fit)
              if (deriv == 1) pred.mod$fit <- pred.mod.0 * pred.mod$fit
            } else {
+             basis.mod.0 <- basis.0
              basis.mod.1 <- basis.mod
              if (!is.null(eval.w)) {
                basis.mod.0 <- cbind(basis.0, outer(rep(1, nrow(basis.0)), eval.w))
@@ -997,6 +1017,7 @@ binstest <- function(y, x, w=NULL, data=NULL, estmethod="reg", family=gaussian()
            if (deriv == 0) pred.mod$fit <- linkinv(pred.mod$fit)
            if (deriv == 1) pred.mod$fit <- pred.mod.0 * pred.mod$fit
          } else {
+           basis.mod.0 <- basis.0
            basis.mod.1 <- basis.mod
            if (!is.null(eval.w)) {
              basis.mod.0 <- cbind(basis.0, outer(rep(1, nrow(basis.0)), eval.w))
